'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _nodentRuntime = require('nodent-runtime');

var _nodentRuntime2 = _interopRequireDefault(_nodentRuntime);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _conventionalChangelog = require('conventional-changelog');

var _conventionalChangelog2 = _interopRequireDefault(_conventionalChangelog);

var _conventionalRecommendedVersion = require('conventional-recommended-version');

var _conventionalRecommendedVersion2 = _interopRequireDefault(_conventionalRecommendedVersion);

var _when = require('when');

var _when2 = _interopRequireDefault(_when);

var _childProcess = require('./lib/child-process');

var childProcess = _interopRequireWildcard(_childProcess);

var _fs = require('./lib/fs');

var fs = _interopRequireWildcard(_fs);

var _log = require('./lib/log');

var log = _interopRequireWildcard(_log);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var bumpVersion = function bumpVersion(_ref) {
  var directory = _ref.directory,
      nextVersion = _ref.nextVersion;
  return childProcess.exec('npm version ' + nextVersion + ' --no-git-tag-version --force', { cwd: directory }).catch(function (err) {
    log.bug('failed to run "npm version" with version "' + nextVersion + '"', err);
    process.exit(1);
  });
};

var tagExists = function tagExists(tagName) {
  return childProcess.exec('git tag --list ' + tagName).then(function (_ref2) {
    var stdout = _ref2.stdout;
    return stdout === tagName;
  });
};

var abortOnExistingTag = function abortOnExistingTag(_ref3) {
  var force = _ref3.force,
      nextVersion = _ref3.nextVersion,
      tagRelease = _ref3.tagRelease;
  return _when2.default.resolve().then(function () {
    return new Promise(function ($return, $error) {
      return new Promise(function ($return, $error) {
        var $logicalAnd_1;if ($logicalAnd_1 = tagRelease && !force) {
          return tagExists(nextVersion).then(function ($await_6) {
            $logicalAnd_1 = $await_6;return $If_3.call(this);
          }.$asyncbind(this, $error), $error);
        }
        function $If_3() {
          return $return($logicalAnd_1);
        }

        return $If_3.call(this);
      }.$asyncbind(this)).then(function ($await_7) {
        if ($await_7) {
          log.error('the tag ' + nextVersion + ' already exists');
          process.exit(1);
        }
        return $return();
      }.$asyncbind(this, $error), $error);
    }.$asyncbind(this));
  });
};

var abortOnUnchangedVersion = function abortOnUnchangedVersion(_ref4) {
  var manifestPath = _ref4.manifestPath,
      nextVersion = _ref4.nextVersion;
  return _when2.default.resolve().then(function () {
    var _require = require(manifestPath),
        currentVersion = _require.version;

    if (nextVersion === currentVersion) {
      log.error('the current version is already ' + nextVersion);
      process.exit(1);
    }
  });
};

var commitChanges = function commitChanges(_ref5) {
  var directory = _ref5.directory,
      nextVersion = _ref5.nextVersion,
      skipHooks = _ref5.skipHooks;

  var baseArgs = ['commit', '-m', 'chore(release): ' + nextVersion];
  var args = skipHooks ? baseArgs.concat('--no-verify') : baseArgs;
  return childProcess.spawn('git', args, { cwd: directory });
};

var getVersion = function getVersion(_ref6) {
  var directory = _ref6.directory,
      postfix = _ref6.postfix;
  return _when2.default.promise(function (resolve, reject) {
    return (0, _conventionalRecommendedVersion2.default)({ directory: directory, postfix: postfix }, function (err, version) {
      return err ? reject(err) : resolve(version);
    });
  });
};

var stageChanges = function stageChanges(directory) {
  return childProcess.exec('git add . -A', { cwd: directory }).catch(function (err) {
    log.bug('failed to stage changes for release commit', err);
    process.exit(1);
  });
};

var tagCommit = function tagCommit(_ref7) {
  var directory = _ref7.directory,
      force = _ref7.force,
      nextVersion = _ref7.nextVersion;
  return childProcess.exec('git tag ' + nextVersion + (force ? ' --force' : ''), { cwd: directory });
};

var generateChangelog = function generateChangelog(_ref8) {
  var changelogPath = _ref8.changelogPath,
      directory = _ref8.directory;
  return fs.unlink(changelogPath).then(function () {
    return _when2.default.promise(function (resolve, reject) {
      var fileWrite$ = fs.createWriteStream(changelogPath);
      fileWrite$.on('finish', function (err) {
        return err ? reject(err) : resolve();
      });
      (0, _conventionalChangelog2.default)({ path: directory, preset: 'angular', releaseCount: 0 }).pipe(fileWrite$);
    });
  }).catch(function (err) {
    log.bug('failed to generate changelog', err);
    process.exit(1);
  });
};

var generateDependencyReport = function generateDependencyReport(_ref9) {
  var dependencyLogPath = _ref9.dependencyLogPath,
      manifestPath = _ref9.manifestPath;
  return childProcess.spawn('node', [require.resolve('package-json-to-readme'), '--no-footer', manifestPath]).then(function (_ref10) {
    var stdout = _ref10.stdout;
    return fs.writeFile(dependencyLogPath, stdout, { encoding: 'utf8' });
  }).catch(function (err) {
    log.bug('failed to generate dependency report', err);
    process.exit(1);
  });
};

exports.default = function (_ref11) {
  return new Promise(function ($return, $error) {
    var directory, force, postfix, skipHooks, tagRelease, version, nextVersion, changelogPath, dependencyLogPath, manifestPath;
    directory = _ref11.directory, force = _ref11.force, postfix = _ref11.postfix, skipHooks = _ref11.skipHooks, tagRelease = _ref11.tagRelease, version = _ref11.version;
    return new Promise(function ($return, $error) {
      var $logicalOr_2;if (!($logicalOr_2 = version)) {
        return getVersion({ directory: directory, postfix: postfix }).then(function ($await_8) {
          $logicalOr_2 = $await_8;return $If_4.call(this);
        }.$asyncbind(this, $error), $error);
      }
      function $If_4() {
        return $return($logicalOr_2);
      }

      return $If_4.call(this);
    }.$asyncbind(this)).then(function ($await_9) {
      nextVersion = $await_9;
      changelogPath = _path2.default.resolve(directory, 'CHANGELOG.md');
      dependencyLogPath = _path2.default.resolve(directory, 'DEPENDENCIES.md');
      manifestPath = _path2.default.resolve(directory, 'package.json');

      return abortOnUnchangedVersion({ manifestPath: manifestPath, nextVersion: nextVersion }).then(function ($await_10) {
        return abortOnExistingTag({ force: force, nextVersion: nextVersion, tagRelease: tagRelease }).then(function ($await_11) {
          return bumpVersion({ directory: directory, nextVersion: nextVersion }).then(function ($await_12) {
            log.info('version: ' + nextVersion);

            return generateChangelog({ changelogPath: changelogPath, directory: directory }).then(function ($await_13) {
              log.info('changelog: ' + changelogPath);

              return generateDependencyReport({ dependencyLogPath: dependencyLogPath, manifestPath: manifestPath }).then(function ($await_14) {
                log.info('dependency report: ' + dependencyLogPath);

                return stageChanges(directory).then(function ($await_15) {
                  return commitChanges({ directory: directory, nextVersion: nextVersion, skipHooks: skipHooks }).then(function ($await_16) {
                    log.info('release committed');

                    if (tagRelease) {
                      return tagCommit({ directory: directory, force: force, nextVersion: nextVersion }).then(function ($await_17) {
                        log.info('release tagged');
                        return $If_5.call(this);
                      }.$asyncbind(this, $error), $error);
                    }

                    function $If_5() {
                      log.success('complete');
                      return $return();
                    }

                    return $If_5.call(this);
                  }.$asyncbind(this, $error), $error);
                }.$asyncbind(this, $error), $error);
              }.$asyncbind(this, $error), $error);
            }.$asyncbind(this, $error), $error);
          }.$asyncbind(this, $error), $error);
        }.$asyncbind(this, $error), $error);
      }.$asyncbind(this, $error), $error);
    }.$asyncbind(this, $error), $error);
  }.$asyncbind(this));
};